; kleiner-graph.scm
; Stellt zum kleinen Graphen die Daten und Zugriffsfunktionen bereit.

;;; Anmerkung: Das Vorgehen, Daten mit ihren jeweiligen Zugriffsfunktionen
;;; zu koppeln entspricht eigentlich objekt-orientierter Denkweise.
;;; Das ist hier jedoch nicht so umgesetzt.

;;; ===== Kantenliste zu kleiner-graph =====
(define
  kanten
  '((A B 27) (A E 18)
    (B A 27) (B C 12) (B F 12)
    (C B 12) (C D 18) (C F 16)
    (D C 18) (D E 35) (D F 16)
    (E A 18) (E D 35) (E F 25)
    (F B 12) (F C 16) (F D 16) (F E 25))
  )

;;; ===== Zugriffsfunktionen zu Kanten =====
; ----- gib-kante -----
; Gibt zu zwei Knoten und dem Graphen [Kantenliste] die Kante zurueck.
; Die Funktion gibt bei fehlender Kante eine leere Liste zurueck,
; ist damit gegen fehlerhafte Knoteneingaben abgesichert.
(define
  (gib-kante knoten-1 knoten-2 kantenliste)
  (cond
    [(null? kantenliste)
     '()]
    [(and
      (equal? knoten-1 (first (first kantenliste)))
      (equal? knoten-2 (first (rest (first kantenliste)))))
     (first kantenliste)]
    [else
     (gib-kante knoten-1 knoten-2 (rest kantenliste))]
    )
  )

; ----- gib-kanten -----
; Gibt zu einem Ausgangsknoten und dem Graphen alle Kanten zurueck.
; Die Funktion gibt bei fehlendem Knoten eine leere Liste zurueck,
; ist damit gegen fehlerhafte Knoteneingaben abgesichert.
(define
  (gib-kanten knoten kantenliste)
  (cond
    [(null? kantenliste)
     '()]
    [(equal? knoten (first (first kantenliste)))
     (cons
      (first kantenliste)
      (gib-kanten knoten (rest kantenliste)))]
    [else
     (gib-kanten knoten (rest kantenliste))]
    )
  )

;;; ===== Knotenliste zu kleiner-Graph =====
; Typ Assoziationsliste
(define
  knoten
  '((A (B 27) (E 18))
    (B (A 27) (C 12) (F 12))
    (C (B 12) (D 18) (F 16))
    (D (C 18) (E 35) (F 16))
    (E (A 18) (D 35) (F 25))
    (F (B 12) (C 16) (D 16) (E 25)))
  )

; ----- hole-kante -----
; Gibt zu zwei Knoten und dem Graphen [Knotenliste] die Kante zurueck.
; Die Funktion gibt bei fehlender Kante eine leere Liste zurueck,
; ist damit gegen fehlerhafte Knoteneingaben abgesichert.
(define
  (hole-kante von nach assoziationsliste)
  (if
   (and
    (assoc von assoziationsliste)
    (assoc nach (rest (assoc von assoziationsliste))))
   (cons
    von
    (assoc
     nach
     (rest
      (assoc von assoziationsliste))))
   '())
  )

; ----- hole-nachfolger -----
; Gibt zu einem Knoten und dem Graphen [Knotenliste] eine Liste
; der Nachfolgeknoten zurueck.
; Die Funktion gibt bei fehlendem Knoten eine leere Liste zurueck,
; ist damit gegen fehlerhafte Knoteneingaben abgesichert.
(define
  (hole-nachfolger von assoziationsliste)
  (rest
   (cond
     [(assoc von assoziationsliste)]
     [else '(Muell)]
     )
   )
  )
